/*
 * $Id: kebook.cpp,v 1.31 2000/05/25 11:15:10 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#include "kebook.h"
#include "kebookwidget.h"
#include "selectsubbookdlg.h"
#include "selectwaytosearchdlg.h"

#include <qkeycode.h>

#include <kfm.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kmsgbox.h>
#include <kstatusbar.h>
#include <kpopmenu.h>
#include <kaccel.h>

//
// KEBook class member functions
//
void KEBook::getCommonToolbarDir(QString& dir)
{
    char sep = QDir::separator();
    dir = kapp->kde_toolbardir()+sep;
}
void KEBook::getKEBookToolbarDir(QString& dir)
{
    char sep = QDir::separator();
    dir = kapp->kde_datadir()+sep+"kebook"+sep+"pics"+sep;
}
void KEBook::createMainMenu()
{
    QPixmap icon;
    QString common_toolbar_dir, kebook_toolbar_dir;
    getCommonToolbarDir(common_toolbar_dir);
    getKEBookToolbarDir(kebook_toolbar_dir);
    
    // create a popup menu -- in this case, the File menu
    QPopupMenu* file = new QPopupMenu;
    icon.load(common_toolbar_dir+EXIT_TBBTN_PIXMAP);
    file->insertItem(icon, i18n("&Quit"), this, SLOT(quitKEBook()),
                     CTRL+Key_Q, ID_MENUITEM_QUIT);
    
    // put our newly created menu into the main menu bar
    menuBar()->insertItem(i18n("&File"), file, ID_MENU_FILE);

    QPopupMenu* edit = new QPopupMenu();
    edit->insertItem(i18n("&Find"),
                     view, SLOT(slotFindWordInPage()),
                     CTRL+SHIFT+Key_F2,
                     ID_MENUITEM_FIND);
    edit->insertItem(i18n("Find &Next"),
                     view, SLOT(slotFindNextWordInPage()),
                     CTRL+SHIFT+Key_F3,
                     ID_MENUITEM_FINDNEXT);
    menuBar()->insertItem(i18n("&Edit"), edit, ID_MENU_EDIT);

    // Menu ``Show''
    QPopupMenu* show = new QPopupMenu();
    icon.load(kebook_toolbar_dir+MENU_TBBTN_PIXMAP);
    show->insertItem(icon, i18n("&Menu"),
                     this, SLOT(showDictMenu()), 0,
                     ID_MENUITEM_DICT_MENU);
    show->setItemEnabled(ID_MENUITEM_DICT_MENU, false);
    
    icon.load(kebook_toolbar_dir+COPYRIGHT_TBBTN_PIXMAP);
    show->insertItem(icon, i18n("&Copyright"),
                     this, SLOT(showDictCopyright()), 0,
                     ID_MENUITEM_COPYRIGHT);
    show->setItemEnabled(ID_MENUITEM_COPYRIGHT, false);
    menuBar()->insertItem(i18n("&Show"), show, ID_MENU_SHOW);

    // Menu ``Go''
    QPopupMenu* go = new QPopupMenu();
    icon.load(common_toolbar_dir+BACK_TBBTN_PIXMAP);
    go->insertItem(icon, i18n("&Back"), view, SLOT(onBack()),
                   0, ID_MENUITEM_BACK);
    go->setItemEnabled(ID_MENUITEM_BACK, false);
    
    icon.load(common_toolbar_dir+FORWARD_TBBTN_PIXMAP);
    go->insertItem(icon, i18n("&Forward"), view, SLOT(onForward()),
                   0, ID_MENUITEM_FORWARD);
    go->setItemEnabled(ID_MENUITEM_FORWARD, false);
    menuBar()->insertItem(i18n("&Go"), go, ID_MENU_GO);

    // Menu ``Option''
    QPopupMenu* setup = new QPopupMenu;
    icon.load(kebook_toolbar_dir+WAY_TO_SEARCH_TBBTN_PIXMAP);
    setup->insertItem(icon, i18n("Select a &Way to search..."),
                      this, SLOT(openSelectWayToSearchDlg()),
                      CTRL+SHIFT+Key_W,
                      ID_MENUITEM_OPEN_SELECT_WAY);

    icon.load(common_toolbar_dir+SETSUBBOOK_TBBTN_PIXMAP);
    setup->insertItem(icon, i18n("&Select a Dictionary..."),
                      this, SLOT(openSelectSubbookDlg()),
                      CTRL+SHIFT+Key_S,
                      ID_MENUITEM_OPEN_SELECT_DICT_DLG);
    setup->setItemEnabled(ID_MENUITEM_OPEN_SELECT_DICT_DLG, false);

    icon.load(common_toolbar_dir+OPENDIR_TBBTN_PIXMAP);
    setup->insertItem(icon,
                      i18n("Set a &Directory to contain the dictionary..."),
                      this, SLOT(openDictPathDlg()),
                      CTRL+SHIFT+Key_D,
                      ID_MENUITEM_OPEN_SET_DIR_DLG);

    icon.load(kebook_toolbar_dir+KEYBOARD_TBBTN_PIXMAP);
    setup->insertItem(icon,
                      i18n("Edit Keyboard accelerators..."),
                      view, SLOT(openKeyDlg()),
                      CTRL+SHIFT+Key_A,
                      ID_MENUITEM_OPEN_EDIT_ACCEL_DLG);
    menuBar()->insertItem(i18n("&Options"), setup, ID_MENU_OPTION);

    // Menu ``CD-ROM''
    if (rescanDictionariesInCDROM()) {
        QPopupMenu* cdrom = new QPopupMenu();
        icon.load(kebook_toolbar_dir+DICTIONARIES_TBBTN_PIXMAP);
        cdrom->insertItem(icon, i18n("Select dictionary"),
                          m_dictionary_list_popup);
        this->connect(m_dictionary_list_popup,
                SIGNAL(activated(int)), SLOT(onSelectDictionaryFromList(int)));
        this->connect(m_function_key,
                      SIGNAL(activated(int)),
                      SLOT(onSelectDictionaryFromList(int)));
        icon.load(common_toolbar_dir+RELOAD_TBBTN_PIXMAP);
        cdrom->insertSeparator();
        cdrom->insertItem(icon, i18n("Rescan dictionaries"), this,
                          SLOT(rescanDictionariesInCDROM()),
                          CTRL+SHIFT+Key_R, ID_MENUITEM_RESCAN_DICTIONARIES);
        menuBar()->insertItem(i18n("&CD-ROM"), cdrom, ID_MENU_CDROM);
    } else {
        menuBar()->insertItem(i18n("&CD-ROM"), ID_MENU_CDROM);
        menuBar()->setItemEnabled(ID_MENU_CDROM, false);
    }
    
    menuBar()->insertSeparator();
    
    // Menu ``HELP''
    // KDE will generate a short help menu automagically
    QString about_message = "KEBook --- Version "VERSION"\n\n";

    const char* COPYRIGHT_MESSAGE =
        i18n("Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>\n" 
             "KEBook is an application with which you look up word \n" 
             "in your CD-ROM books of EB, EBG, EBXA, EBXA-C, S-EBXA and EPWING formats \n" 
             " \n" 
             "This program is free software; you can redistribute it and/or modify \n" 
             "it under the terms of the GNU General Public License as published by \n" 
             "the Free Software Foundation; either version 2, or (at your option) \n" 
             "any later version. \n" 
             " \n" 
             "This program is distributed in the hope that it will be useful, \n" 
             "but WITHOUT ANY WARRANTY; without even the implied warranty of \n" 
             "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n" 
             "GNU General Public License for more details.\n");
    
    QPopupMenu* help = kapp->getHelpMenu(true, about_message+COPYRIGHT_MESSAGE);
    menuBar()->insertItem(i18n("&Help"), help, ID_MENU_HELP);
}
void KEBook::createToolBar()
{
    // insert two buttons "Forward & Back"
    char sep = QDir::separator();
    QString path = kapp->kde_toolbardir()+sep;
    QPixmap pixmap;
    pixmap.load(path+BACK_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_BACK, SIGNAL(clicked()),
                            view, SLOT(onBack()), false, 
                            i18n("Back history"));
    
    pixmap.load(path+FORWARD_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_FORWARD, SIGNAL(clicked()),
                            view, SLOT(onForward()), false, 
                            i18n("Forward history"));

    pixmap.load(path+STOP_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_STOPLOOKINGUP, SIGNAL(clicked()),
                            kapp, SLOT(stopLookingUp()), false,
                            i18n("Stop looking up"));
    
    pixmap.load(path+SETSUBBOOK_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_SETSUBBOOK, SIGNAL(clicked()),
                            this, SLOT(openSelectSubbookDlg()), false,
                            i18n("Select a Dictionary"));

    pixmap.load(path+OPENDIR_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_OPENDIR, SIGNAL(clicked()),
                            this, SLOT(openDictPathDlg()), true,
                            i18n("Set a directory to contain the dictionary"));

    getKEBookToolbarDir(path);
    pixmap.load(path+MENU_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_MENU, SIGNAL(clicked()),
                            this, SLOT(showDictMenu()), false,
                            i18n("Menu"));
    pixmap.load(path+COPYRIGHT_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_COPYRIGHT, SIGNAL(clicked()),
                            this, SLOT(showDictCopyright()), false,
                            i18n("Copyright"));
    
    QPopupMenu* way_to_search_popup = new QPopupMenu;
    KAccel* accel = new KAccel(this);
    vector<WayToSearchTableElement>::iterator it;
    it = m_way_to_search_table->begin();
    while (it != m_way_to_search_table->end()) {
        pixmap.load(path+it->getIconFile());
        QString descript = it->getDescription();
        descript += "\t";
        descript += keyToString(it->getAccel());
        way_to_search_popup->insertItem(pixmap, descript,
                                        &(*it), SLOT(setWayToSearch()));
        accel->insertItem(it->getDescription(), it->getAction(), it->getAccel());
        accel->connectItem(it->getAction(), &(*it), SLOT(setWayToSearch()));
        ++it;
    }

    pixmap.load(path+WAY_TO_SEARCH_TBBTN_PIXMAP);
    toolBar()->insertButton(pixmap, ID_TBBTN_WAY_TO_SEARCH,
                            way_to_search_popup, true, i18n("Choose a way to search"));
    updateSelectionOfWayToSearch();
}
void KEBook::createStatusBar()
{
    KStatusBar* status = statusBar();
    status->setBorderWidth(5);
    
    m_pb_lookingup = new KProgress(0, 100, 0,
                                   KProgress::Horizontal, status,
                                   "Looking up progress");
    CHECK_PTR(m_pb_lookingup);
    m_pb_lookingup->setBarStyle(KProgress::Blocked);
    m_pb_lookingup->setTextEnabled(true);
    m_pb_lookingup->setBackgroundColor(status->backgroundColor());
    status->insertWidget(m_pb_lookingup, 100,  IDS_LOOKINGUP_PROGRESSIVE);

    m_lbl_message = new QLabel(i18n("Unselected Dictionary"), status);
    status->insertWidget(m_lbl_message, 0, IDS_STATUS_MESSAGE);

    enableStatusBar();
}
KEBook::KEBook()
    :  KTMainWindow::KTMainWindow(),
       view(new KEBookWidget(this)),
       m_dictionary_list_popup(new QPopupMenu)
{
    // Tell the KTMainWindow that this is indeed the main widget
    setView(view);

    // QAccel
    m_function_key = new QAccel(view);
    for (int index = 0; Key_F2+index <= Key_F12; index++) {
        m_function_key->insertItem(Key_F2+index, index);
    }
    
    WayToSearchTableElement
        forward_match(this, KEBookApp::FRONT, ID_MENUITEM_SELECT_FORWARD_MATCH,
                      i18n("Forward match"), "Forward match",
                      CTRL+SHIFT+Key_F,
                      MATCH_FORWARD_TBBTN_PIXMAP),
        exact_match(this, KEBookApp::EXACT, ID_MENUITEM_SELECT_EXACT_MATCH,
                    i18n("Exact match"), "Exact match",
                    CTRL+SHIFT+Key_E,
                    MATCH_EXACT_TBBTN_PIXMAP),
        backward_match(this, KEBookApp::TAIL, ID_MENUITEM_SELECT_BACKWARD_MATCH,
                       i18n("Backward match"), "Backward match",
                       CTRL+SHIFT+Key_T,
                       MATCH_BACKWARD_TBBTN_PIXMAP);
    m_way_to_search_table = new vector<WayToSearchTableElement>;
    m_way_to_search_table->push_back(forward_match);
    m_way_to_search_table->push_back(exact_match);
    m_way_to_search_table->push_back(backward_match);
    
    // Create menu
    createMainMenu();
    // Create tool bar
    createToolBar();
    // Create status bar
    createStatusBar();

    connect(getApp(), SIGNAL(sigReleaseSubbook()),
            SLOT(onReleaseSubbook()));
    connect(getApp(), SIGNAL(sigSetDictPath()),
            SLOT(onSetDictPath()));
    connect(getApp(), SIGNAL(sigSetSubbook()),
            SLOT(onSetSubbook()));
    QObject::connect(this, SIGNAL(sigCloseMainWindow()),
                     getApp(), SLOT(quit()));

    // Set caption
    setCaption(PACKAGE);
}

KEBook::~KEBook()
{
    delete m_way_to_search_table;
}
void KEBook::show()
{
    readConfig();
    KTMainWindow::show();
    
    // Geometry
    KConfig* config = kapp->getConfig();
    config->setGroup(CFG_GEOMETRY_GROUP);
    const QRect default_rect(0, 35, 600, 635);
    setGeometry(config->readRectEntry(CFG_MAIN_WINDOW_ENTRY,
                                      &default_rect));
}
void KEBook::closeEvent(QCloseEvent* ev)
{
    saveConfig();
    emit sigCloseMainWindow();
    KTMainWindow::closeEvent(ev);
}
void KEBook::keyPressEvent(QKeyEvent* ev)
{
    if (ev->key() == Key_Escape) {
        getApp()->stopLookingUp();
    }
}
void KEBook::saveProperties(KConfig *config)
{
    config->setGroup(CFG_GEOMETRY_GROUP);
    QRect rect(x(), y(), width(), height());
    config->writeEntry(CFG_MAIN_WINDOW_ENTRY, rect);
    
}
void KEBook::readProperties(KConfig *config)
{
    // Path
    config->setGroup(CFG_PATH_GROUP);
    QString dict_path =
        config->readEntry(CFG_DICT_PATH_ENTRY,
                          DEFAULT_DICT_PATH);
    if (KEBookApp::isAvailableDictPath(dict_path)) {
        getApp()->setDictPath(dict_path);
    } else {
        return ;
    }
    KEBookApp::SubbookInfoListIterator it;
    it = getApp()->getBeginOfSubbookInfoList();
    try {
        if (it != getApp()->getEndOfSubbookInfoList()) {
            config->setGroup(CFG_GENERAL_GROUP);
            int i = config->readNumEntry(CFG_SUBBOOK_PREVNUM_ENTRY,
                                         DEFAULT_PREV_SUBBOOK);
            if (i != DEFAULT_PREV_SUBBOOK) {
                QString dir;
                dir = config->readEntry(CFG_SUBBOOK_PREVDIR_ENTRY,
                                        DEFAULT_PREV_SUBBOOK_DIR);
                if (dir == (*(it+i)).directory.c_str()) {
                    setSubbook(i);
                }
            }
        } else if (++it == getApp()->getEndOfSubbookInfoList()){
            setSubbook(0);
        }
    } catch (ebook::CEBook_Exception& e) {
        cerr << e.what() << endl;
    }
}
void KEBook::setSubbook(int i)
{
    getApp()->setSubbookWithIndex(i);
    
    showDictCopyright();
    setCaption(getApp()->getSubbookTitle());
    view->clearMatchWordsList();
    m_lbl_message->setText(i18n("Ready"));
    emit sigSelectedSubbook();
}
void KEBook::setSubbook(const KEBookApp::DictSpec& dictspec)
{
    getApp()->setDictionary(dictspec);
    
    showDictCopyright();
    setCaption(getApp()->getSubbookTitle());
    view->clearMatchWordsList();
    m_lbl_message->setText(i18n("Ready"));
    emit sigSelectedSubbook();
}
void KEBook::openSelectWayToSearchDlg()
{
    SelectWayToSearchDlg* dlg =
        new SelectWayToSearchDlg(getApp()->getWayToSearch(),
                                 this, "SelectWayToSeachDlg");
    if (dlg->exec()) {
        getApp()->setWayToSearch(dlg->getWayToSearch());
    }
    delete dlg;
}
void KEBook::openSelectSubbookDlg()
{
    vector<string> list;
    getApp()->getSubbookTitleList(list);

    int selected_subbook = getApp()->getSubbook();
    SelectSubbookDlg* dlg = new SelectSubbookDlg(list, selected_subbook, this);

    if (dlg->exec() &&
        dlg->getSelectedSubbook() != selected_subbook) {
        setSubbook(dlg->getSelectedSubbook());
    }
    delete dlg;
}
void KEBook::openDictPathDlg(const char* dict_path)
{
    KDirDialog* dlg = new KDirDialog(dict_path, this, "DictDirDlg");
    
    dlg->setCaption(i18n("Select a directory to contain the dictionary."));
    dlg->show();
    if (dlg->result() == KDirDialog::Accepted) {
        QString new_dict_path = dlg->selectedFile();
        if (!new_dict_path.isEmpty()) {
            if (KEBookApp::isAvailableDictPath(new_dict_path)) {
                getApp()->setDictPath(new_dict_path);
            } else {
                KMsgBox::message((QWidget *)this, PACKAGE "--Error",
                                 i18n("The directory which you select now "
                                      "seems not to contain the dictionaries."),
                                 KMsgBox::EXCLAMATION | KMsgBox::DB_FIRST);
                openDictPathDlg(dict_path);
                delete dlg;
                return;
            }
            KEBookApp::SubbookInfoListIterator it;
            it = getApp()->getBeginOfSubbookInfoList();
            if (++it == getApp()->getEndOfSubbookInfoList()) {
                setSubbook(0);
            }
            onSetDictPath();
        }
    }
    delete dlg;
}
void KEBook::preLookupWord(const char* word)
{
    emit sigSuspendGUIParts();
    toolBar()->setItemEnabled(ID_TBBTN_STOPLOOKINGUP, true);
    string message = i18n("Now Looking up ");
    message += word;
    message += i18n("...");
    m_lbl_message->setText(message.c_str());
}
void KEBook::postLookupWord()
{
    m_pb_lookingup->setValue(0);
    m_pb_lookingup->update();
    m_lbl_message->setText(i18n("Ready"));
    emit sigResumeGUIParts();
    toolBar()->setItemEnabled(ID_TBBTN_STOPLOOKINGUP, false);
}
void KEBook::showDictMenu()
{
    getApp()->clearCurResult();
    string menu;
    getApp()->getDictMenu(menu);
    view->printResult("Menu", menu);
}
void KEBook::onReleaseSubbook()
{
    m_lbl_message->setText(i18n("Unselected Dictionary"));
}
void KEBook::onSetSubbook()
{
    menuBar()->setItemEnabled(ID_MENUITEM_COPYRIGHT, true);
    toolBar()->setItemEnabled(ID_TBBTN_COPYRIGHT, true);
    menuBar()->setItemEnabled(ID_MENUITEM_DICT_MENU, true);
    toolBar()->setItemEnabled(ID_TBBTN_MENU, true);
}
void KEBook::onSetDictPath()
{
    menuBar()->setItemEnabled(ID_MENUITEM_OPEN_SELECT_DICT_DLG, true);
    toolBar()->setItemEnabled(ID_TBBTN_SETSUBBOOK, true);
}
void KEBook::showDictCopyright()
{
    getApp()->clearCurResult();
    string copyright;
    getApp()->getCopyright(copyright);
    view->printResult("Copyright", copyright);
}
bool KEBook::event(QEvent* ev)
{
    switch (ev->type()) {
    case Event_LookupFinish:
        view->finishLookup();
        postLookupWord();
        break;
    case Event_Progress: {
        KEBookProgressEvent* prog_ev
            = (KEBookProgressEvent*)ev;
        m_pb_lookingup->setValue(prog_ev->getPercent());
        emit sigAddMatchWordsList(prog_ev->getIndex(),
                                  prog_ev->getCount());
        break;
    }
    default:
        KTMainWindow::event(ev);
        break;
    }
    return false;
}
void KEBook::tellWayToSearchToEngine(int menu_id)
{
    switch (menu_id) {
    case ID_MENUITEM_SELECT_FORWARD_MATCH:
        getApp()->setWayToSearch(KEBookApp::FRONT);
        break;
    case ID_MENUITEM_SELECT_EXACT_MATCH:
        getApp()->setWayToSearch(KEBookApp::EXACT);
        break;
    case ID_MENUITEM_SELECT_BACKWARD_MATCH:
        getApp()->setWayToSearch(KEBookApp::TAIL);
        break;
    }
    updateSelectionOfWayToSearch();
}
void KEBook::updateSelectionOfWayToSearch()
{
    KEBookApp::WAY_TO_SEARCH way_to_search = getApp()->getWayToSearch();
    QPixmap icon;
    QString kebook_toolbar_dir;
    getKEBookToolbarDir(kebook_toolbar_dir);
    icon.load(kebook_toolbar_dir+WAY_TO_SEARCH_TBBTN_PIXMAP);
    vector<WayToSearchTableElement>::iterator it;
    it = m_way_to_search_table->begin();
    while (it != m_way_to_search_table->end()) {
        if (way_to_search == it->getWayToSearch()) {
            menuBar()->setItemChecked(it->getMenuId(), TRUE);
            icon.load(kebook_toolbar_dir+it->getIconFile());
        } else {
            menuBar()->setItemChecked(it->getMenuId(), FALSE);
        }
        it++;
    }
    toolBar()->setButtonPixmap(ID_TBBTN_WAY_TO_SEARCH, icon);
}
bool KEBook::rescanDictionariesInCDROM()
{
    QPixmap icon;
    QString common_toolbar_dir;
    getCommonToolbarDir(common_toolbar_dir);
    icon.load(common_toolbar_dir+SETSUBBOOK_TBBTN_PIXMAP);
    m_dictionary_list_popup->clear();
    if (!getApp()->rescanDictSpecListInCDROM()) {
        return false;
    }
    KEBookApp::DictSpecListIterator it;
    it = getApp()->getBeginOfDictSpecListInCDROM();
    if (it != getApp()->getEndOfDictSpecListInCDROM()) {
        int id = 0;
        while (it != getApp()->getEndOfDictSpecListInCDROM()) {
            QString caption = (*it++).title.c_str()
                +('\t'+keyToString(Key_F2+id));
            m_dictionary_list_popup->insertItem(icon, caption, id);
            id++;
        }
        m_dictionary_list_popup->setEnabled(true);
    } else {
        m_dictionary_list_popup->setEnabled(false);
    }
    return true;
}
void KEBook::onSelectDictionaryFromList(int i)
{
    if (getApp()->getBeginOfDictSpecListInCDROM()+i
            >= getApp()->getEndOfDictSpecListInCDROM()) {
        return;
    }
    KEBookApp::DictSpec spec = *(getApp()->getBeginOfDictSpecListInCDROM()+i);
    if (KEBookApp::isAvailableDictPath(spec.directory.c_str())) {
        setSubbook(spec);
    } else {
        KMsgBox::message(this, PACKAGE "--Error",
                         i18n("Error occurred to me on changing dictionary.\n"
                              "Please check if your CD-ROM is mounted and "
                              "[Rescan dictionaries] from [CD-ROM] menu.\n"),
                         KMsgBox::EXCLAMATION | KMsgBox::DB_FIRST);
    }
}

#include "kebook.moc"

//
// $Log: kebook.cpp,v $
// Revision 1.31  2000/05/25 11:15:10  hige
// o Fix a bug of finding the word in a page.
// o etc...
//
// Revision 1.30  2000/04/29 13:45:42  hige
// *** empty log message ***
//
// Revision 1.29  2000/04/27 17:42:27  hige
// o Add dialog for editing accelarator keybinds.
// o Change INSERT_MARK from '\\' to '\a'
// o etc...
//
// Revision 1.28  2000/04/16 05:29:22  hige
// Correct previous bug fix.
//
// Revision 1.27  2000/04/16 00:24:31  hige
// Correct behavior when CD-ROM is rescanned.
//
// Revision 1.26  2000/04/14 12:12:14  hige
// Replace ``automoc'' with ``am_edit''.
//
// Revision 1.25  2000/04/06 01:54:05  hige
// Show accelerator keybinds of selecting a way to search in toolbar's button.
//
// Revision 1.24  2000/04/06 01:18:03  hige
// Add keyboard accelerator keys.
//
// Revision 1.23  2000/04/04 12:59:21  hige
// Change position of log in comment.
//
//
  
