/*
 * $Id: resultview.cpp,v 1.14 2000/05/25 11:15:10 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <kapp.h>
#include <kclipboard.h>

#include <qkeycode.h>

#include <strstream>

#include "resultview.h"

#define  HTML_TABLE_START               "<table cellspacing=\"0\" cellpadding=\"0\"><tr>"
#define  HTML_TABLE_END                 "</tr></table>"
#define  HTML_TABLE_DATA_START          "<td nowrap>"
#define  HTML_TABLE_DATA_END            "</td>"
#define  HTML_IMG                       "<img "

#define  EMPTY  0

static const int WRAP_DELAY     =   4;

static const QColor& HTML_DEFAULT_TXT_COLOR = black;
static const QColor& HTML_DEFAULT_LNK_COLOR = blue;
static const QColor& HTML_DEFAULT_VLNK_COLOR = cyan;

template <class T>
inline T max(T x, T y)
{
    return (x > y) ? x : y;
}

KResultView::KResultView(int fontsize, QWidget *_parent, const char *_name, int _flags,
                         KResultView *_parent_view)
    : KHTMLView(_parent, _name, _flags, _parent_view),
      m_fontsize(fontsize),
      m_margin(WRAP_DELAY*fontsize),
      m_offset_vert(0)
{
    getKHTMLWidget()->setDefaultBGColor(kapp->backgroundColor);
    getKHTMLWidget()->setDefaultTextColors(kapp->windowTextColor,
                                        HTML_DEFAULT_LNK_COLOR,
                                        HTML_DEFAULT_VLNK_COLOR);
    connect(kapp, SIGNAL(kdisplayPaletteChanged()),
            SLOT(slotChangeWidgetColor()));
}
KResultView::~KResultView()
{
}
void KResultView::slotScrollVert(int y)
{
    m_offset_vert = max(y, 0);
    KHTMLView::slotScrollVert(m_offset_vert);
}
void KResultView::slotScrollVertUp()
{
    QKeyEvent* ev = new QKeyEvent(Event_KeyPress, Key_Up, 0, 0);
    KApplication::postEvent(getKHTMLWidget(), ev);

}
void KResultView::slotScrollVertDown()
{
    QKeyEvent* ev = new QKeyEvent(Event_KeyPress, Key_Down, 0, 0);
    KApplication::postEvent(getKHTMLWidget(), ev);
}
void KResultView::slotPageUp()
{
    QKeyEvent* ev = new QKeyEvent(Event_KeyPress, Key_PageUp, 0, 0);
    KApplication::postEvent(getKHTMLWidget(), ev);
}
void KResultView::slotPageDown()
{
    QKeyEvent* ev = new QKeyEvent(Event_KeyPress, Key_PageDown, 0, 0);
    KApplication::postEvent(getKHTMLWidget(), ev);
}
void KResultView::slotChangeWidgetColor()
{
    getKHTMLWidget()->setDefaultBGColor(kapp->windowColor);
    getKHTMLWidget()->setDefaultTextColors(kapp->textColor,
                                        HTML_DEFAULT_LNK_COLOR,
                                        HTML_DEFAULT_VLNK_COLOR);
    parse();
}
bool KResultView::mousePressedHook(const char* _url, const char *_target,
                                   QMouseEvent *_ev, bool _isselected)
{
    if (url != 0L) {
        switch (_ev->button()) {
        case LeftButton:
            if (_url) {
                KURL u(_url);
                if (strcmp(u.protocol(), "eb") == 0) {
                    long l = strtol(u.host(), EMPTY, 10);
                    emit clickLink((int)l, u.port());
                }
            }
            return true;
        }
    }
    return KHTMLView::mousePressedHook(_url, _target, _ev, _isselected);
}
bool KResultView::mouseReleaseHook(QMouseEvent *_ev)
{
    switch (_ev->button()) {
    case LeftButton:
        {
            QString selectstr;
            getSelectedText(selectstr);
            if (!selectstr.isEmpty()) {
                KApplication::clipboard()->setText((const char*) selectstr);
            }
        }
        break;
    }
    return KHTMLView::mouseReleaseHook(_ev);
}
void wrapText(string& wraptext,
              const char* nowraptext,
              int col,
              const char* row_start,
              const char* row_end)
{
    const size_t maxlen = strlen(nowraptext)+1;
    int nowcol = col;
    bool hyphen = false;
    
    while (*nowraptext != '\0') {
        if (*nowraptext == '<') {
            if (strncmp(nowraptext+1, "br", strlen("br")) == 0
                || (*(nowraptext+1) == 'p' && !isalpha(*(nowraptext+2)))
                || strncmp(nowraptext+1, "table", strlen("table")) == 0) {
                nowcol = col;
            }
            do {
                wraptext += *nowraptext;
            } while (*nowraptext++ != '>');
        } else {
            int m = mblen(nowraptext, maxlen);
            if (m == 1 && isalpha(*nowraptext) &&
                mblen(nowraptext+m, maxlen) == 1 &&
                isalpha(*(nowraptext+m))) {
                hyphen = true;
            } else {
                hyphen = false;
            }
            nowcol -= m;
            while (m-- > 0) {
                wraptext += *nowraptext++;
            }
        }
        if ((!hyphen && nowcol < 0) || (hyphen && nowcol+WRAP_DELAY < 0)) {
            if (hyphen) {
                wraptext += '-';
                hyphen = false;
            }
            wraptext += row_end;
            wraptext += '\n';
            wraptext += row_start;
            nowcol = col;
        }
    }
}
void KResultView::printContent(const string& title, const string& content,
                               const char* row_start,
                               const char* row_end)
{
    int col = 2*(width()-m_margin)/m_fontsize;
    string wrapped_text;
    wrapText(wrapped_text, content.c_str(), col, row_start, row_end);
    strstream html;
    
    html << "<html>" << endl
         << "<head>" << endl
         << "<title>" << title << "</title>" << endl
         << "</head>" << endl
         << "<body>" << endl
         << wrapped_text
         << "</body>" << endl
         << "</html>" << ends;
        
    begin(title.c_str());
    parse();
    write(html.str());
    end();
    show();
}
#include "resultview.moc"
//
// $Log: resultview.cpp,v $
// Revision 1.14  2000/05/25 11:15:10  hige
// o Fix a bug of finding the word in a page.
// o etc...
//
// Revision 1.13  2000/04/29 16:24:14  hige
// Correct bug about treating QTextStream class.
//
// Revision 1.12  2000/04/27 17:42:27  hige
// o Add dialog for editing accelarator keybinds.
// o Change INSERT_MARK from '\\' to '\a'
// o etc...
//
// Revision 1.11  2000/04/14 12:12:14  hige
// Replace ``automoc'' with ``am_edit''.
//
// Revision 1.10  2000/04/04 12:59:21  hige
// Change position of log in comment.
//
//
